import { ServiceLogger } from "@effect-aws/commons";
import { Effect, FiberRef, Layer } from "effect";
import { dual } from "effect/Function";
import { globalValue } from "effect/GlobalValue";
/**
 * @since 1.0.0
 * @category lambda service config
 */
const currentLambdaServiceConfig = globalValue("@effect-aws/client-lambda/currentLambdaServiceConfig", () => FiberRef.unsafeMake({}));
/**
 * @since 1.0.0
 * @category lambda service config
 */
export const withLambdaServiceConfig = dual(2, (effect, config) => Effect.locally(effect, currentLambdaServiceConfig, config));
/**
 * @since 1.0.0
 * @category lambda service config
 */
export const setLambdaServiceConfig = (config) => Layer.locallyScoped(currentLambdaServiceConfig, config);
/**
 * @since 1.0.0
 * @category adapters
 */
export const toLambdaClientConfig = Effect.gen(function* () {
    const { logger: serviceLogger, ...config } = yield* FiberRef.get(currentLambdaServiceConfig);
    const logger = serviceLogger === true
        ? yield* ServiceLogger.toClientLogger(ServiceLogger.defaultServiceLogger)
        : (serviceLogger ? yield* ServiceLogger.toClientLogger(ServiceLogger.make(serviceLogger)) : undefined);
    return { logger, ...config };
});
//# sourceMappingURL=LambdaServiceConfig.js.map